RAMDISK_DIR=ramdisk-0.2
RAMDISK_ARCHIVE=$(RAMDISK_DIR).tgz
INITSCRIPT_ARCHIVE=initscripts-0.1.tgz
INITSCRIPT_DIR=initscripts-0.1

RAMDISK_IMAGE=$(RAMDISK_DIR)/ramdisk.ext2.gz.uboot

.PHONY: ramdisk ramdiskfs initscripts

ramdisk:	$(RAMDISK_DIR)/ramdisk.ext2.gz.uboot

ramdiskfs:	$(RAMDISK_DIR)/.fs

initscripts:	$(INITSCRIPT_DIR)/.unpacked

$(INITSCRIPT_DIR)/.unpacked:	$(INITSCRIPT_ARCHIVE)
	tar xzvf $(INITSCRIPT_ARCHIVE)
	touch $(INITSCRIPT_DIR)/.unpacked

$(RAMDISK_DIR)/.unpacked:	$(RAMDISK_ARCHIVE)
	tar -xzvf $(RAMDISK_ARCHIVE)
	touch $(RAMDISK_DIR)/.unpacked

$(RAMDISK_DIR)/.patched:	$(RAMDISK_DIR)/.unpacked
	touch $(RAMDISK_DIR)/.patched

$(RAMDISK_DIR)/.configured:	$(RAMDISK_DIR)/.patched
	touch $(RAMDISK_DIR)/.configured

$(RAMDISK_DIR)/.fs:	$(RAMDISK_DIR)/.configured $(INITSCRIPT_DIR)/.unpacked
	mkdir -p $(RAMDISK_DIR)/fs
	sudo tar -C $(RAMDISK_DIR)/fs -xzvf $(RAMDISK_DIR)/files.tgz
	sudo tar -C $(RAMDISK_DIR)/fs -xzvf $(RAMDISK_DIR)/dev.tgz
	sudo cp $(INITSCRIPT_DIR)/rc.sh $(RAMDISK_DIR)/fs/etc/rc.sh
	touch $(RAMDISK_DIR)/.fs

$(RAMDISK_DIR)/ramdisk.ext2:	$(RAMDISK_DIR)/.fs busybox-bin.tgz
	sudo tar -C $(RAMDISK_DIR)/fs -xvzf busybox-bin.tgz
	date +"%H:%M:%S %b %d, %Y - Build_DSM-$(BUILD_DSM_VERSION)" > $(RAMDISK_DIR)/fs/version
	( cd $(RAMDISK_DIR) ; genext2fs -d fs -b 6144 ramdisk.ext2 )

$(RAMDISK_DIR)/ramdisk.ext2.gz:	$(RAMDISK_DIR)/ramdisk.ext2
	gzip < $(RAMDISK_DIR)/ramdisk.ext2 > $(RAMDISK_DIR)/ramdisk.ext2.gz

$(RAMDISK_DIR)/ramdisk.ext2.gz.uboot: $(RAMDISK_DIR)/ramdisk.ext2.gz $(MKIMAGE)
	$(MKIMAGE) -n default -A ppc -O linux -T ramdisk -C gzip \
		-d $(RAMDISK_DIR)/ramdisk.ext2.gz \
		$(RAMDISK_DIR)/ramdisk.ext2.gz.uboot

CLEAN_ALL+=clean_ramdisk

clean_ramdisk:
	sudo rm -rf $(RAMDISK_DIR)/fs $(RAMDISK_DIR)/.fs
	rm -f $(RAMDISK_DIR)/ramdisk.ext2
	rm -f $(RAMDISK_DIR)/ramdisk.ext2.gz
	rm -f $(RAMDISK_DIR)/ramdisk.ext2.gz.uboot

clobber_ramdisk:
	sudo rm -rf $(RAMDISK_DIR) $(INITSCRIPT_DIR)
