#include <sys/types.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include "fw.h"

struct stat		statbuf;
struct FW_header	*FW;
char			*buffer;
char			prefix[256] = "firmware";
char			file[256];
char			magic[12] = {
	0x55, 0xaa, 0x53, 0x61, 0x6d, 0x62, 0x61, 0x2d, 0x33, 0x58, 0x55, 0xaa,
};

int main( int argc, char *argv[] )
{
	int		fd, r, i;
	char		*p, opt;
	u_int32_t	c, outputsize = 0;
	u_int32_t	kernelsize = 0, ramdisksize = 0, extrasize = 0;
	int		qflag = 0, pflag = 0, vflag = 0;
	char		*exec_name = 0;
	char		*outputname = 0;
	char		*kernelname = 0;
	char		*ramdiskname = 0;
	char		*extraname = 0;

	exec_name = argv[0];

	while ((opt = getopt(argc, argv, "hqk:r:e:pv")) > 0) {
		switch( opt ) {
		case 'q':	/* quiet, don't print header */
			qflag = 1;
			break;
		case 'k':	/* kernel image file name */
			kernelname = optarg;
			break;
		case 'r':	/* ramdisk image file name */
			ramdiskname = optarg;
			break;
		case 'e':	/* extra section file name */
			extraname = optarg;
			break;
		case 'p':
			pflag = 1;
			break;
		case 'v':
			vflag = 1;
			break;
		default:
		usage:
			fprintf(stderr,
				"Usage: %s [-qp] [-k kname] [-r rname] "
				"[-e ename] <outputfile>\n",
			exec_name );
		}
	}
	outputname = argv[optind];

	outputsize = sizeof( *FW );

	if( !qflag || vflag )
		fprintf( stdout, "%s Version %d.%d\n", exec_name
				, FW_VERSION, FW_REVISION);
	if( vflag )
		exit( 0 );

	if( kernelname )
	{
		if( stat( kernelname, &statbuf ) < 0 )
		{
			fprintf( stderr, "%s: Can't get file size, %s: %s\n",
					exec_name, kernelname,
					strerror( errno ) );
			exit( -1 );
		}
		kernelsize = statbuf.st_size;
		outputsize += kernelsize;
		outputsize += statbuf.st_size % 64; /* pad up to 64 bytes */
	}

	if( ramdiskname )
	{
		if( stat( ramdiskname, &statbuf ) < 0 )
		{
			fprintf( stderr, "%s: Can't get file size, %s: %s\n",
					exec_name, ramdiskname,
					strerror( errno ) );
			exit( -1 );
		}
		ramdisksize = statbuf.st_size;
		outputsize += ramdisksize;
		outputsize += statbuf.st_size % 64; /* pad up to 64 bytes */
	}

	if( extraname )
	{
		if( stat( extraname, &statbuf ) < 0 )
		{
			fprintf( stderr, "%s: Can't get file size, %s: %s\n",
					exec_name, extraname,
					strerror( errno ) );
			exit( -1 );
		}
		extrasize = statbuf.st_size;
		outputsize += extrasize;
		outputsize += statbuf.st_size % 64; /* pad up to 64 bytes */
	}

	if( ( buffer = malloc( outputsize ) ) == NULL )
	{
		fprintf( stderr, "%s: Can't allocate memory %s\n",
				exec_name, strerror( errno ) );
		exit( -1 );
	}

	memset( buffer, 0, outputsize );
	FW = (struct FW_header *)buffer;
	memcpy( FW->magic, magic, sizeof( FW->magic ) );
	FW->productId = 0x4;
	FW->customId = 0x2;
	FW->modelId = 0x1;

	FW->start_1 = sizeof( *FW );
	FW->size_1 = kernelsize;
	FW->start_2 = FW->start_1 + FW->size_1 + (FW->size_1 % 64);
	FW->size_2 = ramdisksize;
	FW->start_3 = FW->start_2 + FW->size_2 + (FW->size_2 % 64);
	FW->size_3 = extrasize;

	if( kernelname )
	{
		if( (fd = open( kernelname, O_RDONLY ) ) < 0 )
		{
			fprintf( stderr, "%s: Can't open %s: %s\n",
					exec_name, kernelname,
					strerror( errno ) );
			exit( -1 );
		}
		for( p = buffer + FW->start_1; (r = read(fd, p, 1024));
				p += r )
			;
		close( fd );
	}

	if( ramdiskname )
	{
		if( (fd = open( ramdiskname, O_RDONLY ) ) < 0 )
		{
			fprintf( stderr, "%s: Can't open %s: %s\n",
					exec_name, ramdiskname,
					strerror( errno ) );
			exit( -1 );
		}
		for( p = buffer + FW->start_2; (r = read(fd, p, 1024));
				p += r )
			;
		close( fd );
	}

	if( extraname )
	{
		if( (fd = open( extraname, O_RDONLY ) ) < 0 )
		{
			fprintf( stderr, "%s: Can't open %s: %s\n",
					exec_name, extraname,
					strerror( errno ) );
			exit( -1 );
		}
		for( p = buffer + FW->start_3; (r = read(fd, p, 1024));
				p += r )
			;
		close( fd );
	}

	FW->start_1 = htonl( FW->start_1 );
	FW->size_1 = htonl( FW->size_1 );
	FW->start_2 = htonl( FW->start_2 );
	FW->size_2 = htonl( FW->size_2 );
	FW->start_3 = htonl( FW->start_3 );
	FW->size_3 = htonl( FW->size_3 );
	
	if ( !qflag )
	{
		fprintf( stdout, "Section 1 %8d, %8d\n", ntohl( FW->start_1 ),
			ntohl( FW->size_1 ) );
		fprintf( stdout, "Section 2 %8d, %8d\n", ntohl( FW->start_2 ),
			ntohl( FW->size_2 ) );
		fprintf( stdout, "Section 3 %8d, %8d\n", ntohl( FW->start_3 ),
			ntohl( FW->size_3 ) );
		fprintf( stdout, "\nMagic      " );
		for( i = 0; i < 12; i++ )
		{
			fprintf( stdout, " 0x%02x", FW->magic[i] );
		}
		fprintf( stdout, "\n" );
		fprintf( stdout,
			"Product ID: 0x%02x, Custom ID: "
			"0x%02x, Model ID: 0x%02x\n\n",
			FW->productId, FW->customId, FW->modelId );

	}
	FW->checksum_1 = cksum_section( buffer, ntohl( FW->start_1 ),
			ntohl( FW->size_1 ) );

	FW->checksum_2 = cksum_section( buffer, ntohl( FW->start_2 ),
			ntohl( FW->size_2 ) );

	FW->checksum_3 = cksum_section( buffer, ntohl( FW->start_3 ),
			ntohl( FW->size_3 ) );

	if( outputname == 0 )
		outputname = "DSM-G600.fw";

	if( (fd = open( outputname, O_WRONLY|O_CREAT, 0666 )) < 0 )
	{
		fprintf( stderr, "%s: Can't open %s: %s\n", exec_name,
				outputname, strerror( errno ) );
		exit( -1 );
	}
	for( p = buffer, c = 0; c < outputsize; c += r, p += r )
	{
		r = outputsize - c;
		if( r > 1024 )
			r = 1024;

		if( (r = write( fd, p, r )) < 0 )
		{
			fprintf( stderr, "%s: write failed %s: %s\n",
					exec_name, extraname,
					strerror( errno ) );
			exit( -1 );
		}
	}
	close( fd );

	exit( 0 );
}
